module adc(
	input 				clk,
	input 				reset,
	
	output wire 		convst,
	output wire 		sck,
	output wire 		mosi,
	input	 				miso,
	
	output reg [47:0]	data_out,
	output reg			data_val,
	input  				data_rdy,
	
	input [31:0]		cfg
);




	wire [1:0]	num_channels;
	
	cfg_reg #(2,4,2'd0) cfg_4(clk,reset,cfg,num_channels); 

	
	localparam S_IDLE  	= 3'd0;
	localparam S_START 	= 3'd1;
	localparam S_CONV  	= 3'd2;
	localparam S_SCK_H 	= 3'd3;
	localparam S_SCK_L 	= 3'd4;
	localparam S_NXT_CH	= 3'd5;
	localparam S_WRITE 	= 3'd6;
	
	reg  [2:0] 	state, state_next;
	reg  [7:0] 	count, count_next;
	
	localparam	mosi_ch0		= 6'b01_000_1;
	localparam	mosi_ch1		= 6'b01_001_1;
	localparam 	mosi_ch2		= 6'b01_100_1;
	localparam 	mosi_ch3		= 6'b01_101_1;
	
	wire 	[1:0]	next_channel;
	reg 	[1:0]	curr_channel, curr_channel_next;
	
	
	reg  	[5:0] 	mosi_data;
	reg  	[11:0]	miso_data;
	wire 	[11:0]	miso_data_next;
	
	reg 				data_val_next;
	wire 	[47:0]	data_out_next;
	
	reg	[11:0]	data_buf 		[3:0];
	reg 	[11:0]	data_buf_next	[3:0];
	
	assign convst = (state == S_START);
	assign sck    = (state == S_SCK_H);
	assign mosi   = ((state == S_SCK_H || state == S_SCK_L) && count < 8'd6) ? 
		mosi_data[count] : mosi_data[0];
	assign miso_data_next = (state == S_SCK_L) ? {miso_data[10:0],miso} : miso_data;
	assign data_out_next = (state == S_WRITE) ? {data_buf[3],data_buf[2],data_buf[1],data_buf[0]} : data_out;
	
	assign next_channel = (curr_channel == num_channels) ? 2'b00 : curr_channel+1;
	
	integer i;

	
	always @(*) begin
		case (next_channel)
			2'b00: mosi_data = mosi_ch0;
			2'b01: mosi_data = mosi_ch1;
			2'b10: mosi_data = mosi_ch2;
			2'b11: mosi_data = mosi_ch3;
		endcase
		if(state == S_WRITE) begin
			data_val_next = 1'b1;
		end else begin
			if(data_val) begin
				data_val_next = !data_rdy;
			end else begin
				data_val_next = 1'b0;
			end
		end
		if(state == S_NXT_CH) begin
				for(i=0; i<4; i=i+1) begin
					data_buf_next[i] = (curr_channel == i) ? miso_data : data_buf[i];
				end
		end else begin
			for(i=0; i<4; i=i+1) begin
				data_buf_next[i] = data_buf[i];
			end
		end
	end
	
	always @(*)begin
		state_next = state;
		count_next = count;
		curr_channel_next = curr_channel;
		case(state)
		S_IDLE: begin
			state_next = S_START;
			count_next = 8'b0;
		end
		S_START: begin
			state_next = (count == 8'd1) ? S_CONV : S_START;
			count_next = count + 1;
		end		
		S_CONV: begin
			if(count==8'd80) begin
			//conversion complete
			state_next = S_SCK_L;
			count_next = 8'd0;
			end else begin
			//else keep waiting
			state_next = S_CONV;
			count_next = count + 1;
			end
		end
		S_SCK_H: begin
			if(count == 8'd11) begin
				state_next = S_NXT_CH;
				count_next = 8'd0;
			end else begin
				state_next = S_SCK_L;
				count_next = count + 1;
			end
		end		
		S_SCK_L: begin
			state_next = S_SCK_H;
		end
		S_NXT_CH: begin
			if(curr_channel >= num_channels) begin
				state_next = S_WRITE;
				curr_channel_next = 2'b0;
			end else begin
				state_next = S_START;
				curr_channel_next = curr_channel + 1;
			end
		end
		S_WRITE: begin
			state_next = S_START;
		end	
		endcase
	end
	
	always @(posedge clk) begin
		if(reset) begin
			state <= S_IDLE;
			count <= 8'd0;
			miso_data <= 12'd0;
			data_out <= 12'd0;
			data_val <= 1'b0;
			curr_channel <= 2'd0;
			for(i=0; i<4; i=i+1) begin
				data_buf[i] <= 12'd0;
			end
		end else begin
			state <= state_next;
			count <= count_next;
			miso_data <= miso_data_next;
			data_out <= data_out_next;
			data_val <= data_val_next;
			curr_channel <= curr_channel_next;
			for(i=0; i<4; i=i+1) begin
				data_buf[i] <= data_buf_next[i];
			end
		end
	end
	
endmodule