module adc_valrdy(
	input  wire clk,
	input  wire reset,
	
	input  wire [31:0] adc_readdata,
	output wire [31:0] adc_writedata,
	output wire [2:0]  adc_address,
	output reg         adc_read,
	output reg         adc_write,
	input  wire        adc_waitrequest,
	
    output reg  [11:0] data,
    output reg         data_val,
    input  wire        data_rdy
);

localparam ST_START_CONV  = 2'd0;
localparam ST_READ_RESULT = 2'd1;
localparam ST_SEND_RESULT = 2'd2;

assign adc_writedata = 32'd0;
assign adc_address = 3'd0;

reg [1:0] state, state_next;
reg [11:0] data_next;

// Outputs
always @(*) begin
	adc_read = 1'b0;
	adc_write = 1'b0;
    data_val = 1'b0;
	
	case (state)
	ST_START_CONV: begin
        adc_write = 1'b1;
	end
	ST_READ_RESULT: begin
		adc_read = 1'b1;
	end
	ST_SEND_RESULT: begin
        data_val = 1'b1;
	end
	default: begin
	end
	endcase
end

// Next state
always @(*) begin
	state_next = state;
    data_next = data;
	
	case (state)
	ST_START_CONV: begin
		if (~adc_waitrequest) state_next = ST_READ_RESULT;
	end
	ST_READ_RESULT: begin
        data_next = adc_readdata[11:0];
		if (~adc_waitrequest) state_next = ST_SEND_RESULT;
	end
    ST_SEND_RESULT: begin
        if (data_rdy) state_next = ST_START_CONV;
    end
	default: begin
		state_next = ST_START_CONV;
	end
	endcase
end

always @(posedge clk) begin
	if (reset) begin
		state <= ST_START_CONV;
        data <= 12'd0;
	end else begin
		state <= state_next;
        data <= data_next;
	end
end

endmodule
