module adc_writer(
	input wire clk,
	input wire reset,
	
	input wire [31:0] adc_readdata,
	output wire [31:0] adc_writedata,
	output wire [2:0] adc_address,
	output reg adc_read,
	output reg adc_write,
	input wire adc_waitrequest,
	
	output reg vga_write,
	output wire [18:0] vga_address,
	output wire [7:0] vga_writedata
);

localparam ST_CLEAR = 2'd0;
localparam ST_START_CONV = 2'd1;
localparam ST_WAIT_CONV = 2'd2;
localparam ST_WAIT_ITER = 2'd3;


localparam X_LIM = 10'd639;
localparam Y_LIM = 9'd479;
localparam TIMER_LIM = 19'd390625;

/*
localparam X_LIM = 10'd7;
localparam Y_LIM = 9'd3;
localparam TIMER_LIM = 19'd10;
*/

reg [8:0] y_coord, y_coord_next;
reg [9:0] x_coord, x_coord_next;
reg [18:0] timer, timer_next;

reg [1:0] state, state_next;
reg use_adc_data;

assign adc_address = 3'd0;
assign adc_writedata = 32'd0;
assign vga_writedata = use_adc_data ? 8'hFF : 8'h00;
assign vga_address = ((use_adc_data ? adc_readdata[8:0] : y_coord) * 640) + x_coord;

// Outputs
always @(*) begin
	adc_read = 1'b0;
	adc_write = 1'b0;
	vga_write = 1'b0;
	use_adc_data = 1'b0;
	
	case (state)
	ST_CLEAR: begin
		vga_write = 1'b1;
	end
	ST_START_CONV: begin
		adc_write = 1'b1;
	end
	ST_WAIT_CONV: begin
		adc_read = 1'b1;
		vga_write = ~adc_waitrequest;
		use_adc_data = 1'b1;
	end
	ST_WAIT_ITER: begin
	end
	default: begin
	end
	endcase
end

// Next state
always @(*) begin
	state_next = state;
	x_coord_next = x_coord;
	y_coord_next = y_coord;
	timer_next = timer;
	
	case (state)
	ST_CLEAR: begin
		if (x_coord == X_LIM) begin
			x_coord_next = 10'd0;
			if (y_coord == Y_LIM) begin
				y_coord_next = 9'd0;
				state_next = ST_START_CONV;
			end else begin
				y_coord_next = y_coord + 9'd1;
			end
		end else begin
			x_coord_next = x_coord + 10'd1;
		end
	end
	ST_START_CONV: begin
		if (~adc_waitrequest) state_next = ST_WAIT_CONV;
	end
	ST_WAIT_CONV: begin
		if (~adc_waitrequest) state_next = ST_WAIT_ITER;
		timer_next = 19'd0;
	end
	ST_WAIT_ITER: begin
		if (timer == TIMER_LIM) begin
			timer_next = 19'd0;
			if (x_coord == X_LIM) begin
				state_next = ST_CLEAR;
				x_coord_next = 10'd0;
			end else begin
				state_next = ST_START_CONV;
				x_coord_next = x_coord + 10'd1;
			end
		end else begin
			timer_next = timer + 19'd1;
		end
	end
	default: begin
		state_next = ST_CLEAR;
	end
	endcase
end

always @(posedge clk) begin
	if (reset) begin
		state <= ST_CLEAR;
		x_coord <= 10'd0;
		y_coord <= 9'd0;
		timer <= 19'd0;
	end else begin
		state <= state_next;
		x_coord <= x_coord_next;
		y_coord <= y_coord_next;
		timer <= timer_next;
	end
end

endmodule
