module time_scaler #(
	parameter width = 8,
	parameter cfg_id = 1
)(
	input  wire             clk,
	input  wire             reset,
	
	input  wire             in_val,
	output wire             in_rdy,
	input  wire [width-1:0] in_msg,
	
	output wire             out_val,
	input  wire             out_rdy,
	output wire [width-1:0] out_msg,
	
	input  wire [31:0]      cfg
);

wire drop;
wire [23:0] max;

// Config register for the sample divider clock
cfg_reg #(24, cfg_id, 24'd0) max_reg (clk, reset, cfg, max);

assign out_msg = in_msg;
assign out_val = in_val & ~drop;
assign in_rdy  = drop | out_rdy;

reg [23:0] count;
assign drop = count != 24'd0;

always @(posedge clk) begin
  if (reset) begin
    count <= 24'd0;
  end else begin
    if (in_val & in_rdy) begin
      if (count >= max) begin
	     count <= 24'd0;
	   end else begin
	     count <= count + 24'd1;
	   end
	 end else begin
	   count <= count;
	 end
  end
end
  
endmodule
